using System.Collections.Generic;

namespace GameFix.Poker
{
    public abstract class DeckBaseLogic
    {
        protected Deck mDeck;

        public void SetDeck(Deck deck)
        {
            mDeck = deck;
        }

        /// <summary>
        /// 初始化方法
        /// </summary>
        public virtual void OnInit() { }
        
        // 创建牌组
        public abstract List<Card> Create();
        
        // 获取牌型检测方法
        public abstract List<CardStyleBaseHelper> GetAllStyleHelper(CardBaseGroup cardGroup);
        
        // 获取牌型比较器
        public abstract IStyleComparer GetStyleComparer();
    }
}